<?
$port = 56170 ;

function LWZ_hexToStr($hex)
{
    $string='';
    for ($i=0; $i < strlen($hex)-1; $i+=2)
    {
        $string .= chr(hexdec($hex[$i].$hex[$i+1]));
    }
    return $string;
}

function LWZ_strToHex($string)
{
    $hex='';
    for ($i=0; $i < strlen($string); $i++)
    {
        $hex .= dechex(ord($string[$i]));
    }
    return $hex;
}

function LWZ_DecodeWord($l,$h)
{
	$l = LWZ_strToHex($l);
	$h = LWZ_strToHex($h);
	$s = (hexdec($h) + hexdec($l) * 256);
	return $s;
}

function LWZ_EncodeWord($int)
{
	if($int > 256) {
  	  $h = chr(0x01);
  	  $l = chr($int - 256);
	} else {
	  $h = chr(0x00);
	  $l = chr($int);
	}
	
	return $h . $l;
}


function LWZ_checksum($str)
{
  $chk = 0;
  for($k = 0; $k < strlen($str); $k = $k +1)
    {
    $o = ord(substr($str,$k,1));
    $chk = $chk + $o;
    }
  $ret = chr($chk % 256);

  return $ret;
}

function LWZ_Escape($str)
{
	$str = str_replace(chr(0x10),chr(0x10).chr(0x10),$str);
	$str = str_replace(chr(0x2B),chr(0x2B).chr(0x18),$str);
	
	return $str;
}

function LWZ_UnEscape($str)
{
	$str = str_replace(chr(0x10).chr(0x10),chr(0x10),$str);
	$str = str_replace(chr(0x2B).chr(0x18),chr(0x18),$str);

	return $str;
}

function LWZ_SendCommand($mode, $command)
{
	global $port;
	
	if($mode == "get") {	$header = chr(0x01) . chr(0x00);};
	if($mode == "set") {	$header = chr(0x01) . chr(0x80);};

	$footer = chr(0x10) . chr(0x03);
	$checksum = LWZ_Escape(LWZ_checksum($header . $command));
	$command = LWZ_Escape($command);

	$dts = $header . $checksum . $command . $footer;

	// Init Comm
	// Try sending text and if it fails, reopen COM Port.
	if(COMPort_SendText($port ,  chr(0x02))===false) { 
	   IPS_LogMessage("LWZ", "COM Port Error. Reopen.");
		COMPort_SetOpen($port,false);
		IPS_ApplyChanges($port);
		COMPort_SetOpen($port,true);
		IPS_ApplyChanges($port);
		COMPort_SendText($port ,  chr(0x02));
	}
	
	IPS_Sleep(10); // Wait fixed time for ack from LWZ, but without really waiting for it
	COMPort_SendText($port ,  $dts);
	IPS_Sleep(1000); // Wait 1 second for the command to get executed
}

function LWZ_Ack()
{
	global $port;
	COMPort_SendText($port ,  chr(0x10));
	//IPS_LogMessage("LWZ", "Send Ack");
}

function LWZ_VerifyChecksum($data)
{
	$header = substr($data,0,2);
	$checksum = substr($data,2,1);
	$reply = substr($data,3,strlen($data)-3-2);
	$checksum_computed = LWZ_Checksum($header . $reply);
	
	return $checksum == $checksum_computed;
}

function LWZ_GetActualValues($lwzdata = "")
{
  $ret = false;
  if(strlen($lwzdata) > 0) {
	  $ret = array();

	  $i = 6; // Outside temperature
	  $ret["OutsideTemperature"] = LWZ_DecodeWord(substr($lwzdata,$i,1),substr($lwzdata,$i+1,1)) / 10;

	  $i = 8; // Flow temperature
	  $ret["FlowTemperature"] = LWZ_DecodeWord(substr($lwzdata,$i,1),substr($lwzdata,$i+1,1)) / 10;

	  $i = 10; // Return temperature
	  $ret["ReturnTemperature"] = LWZ_DecodeWord(substr($lwzdata,$i,1),substr($lwzdata,$i+1,1)) / 10;

	  $i = 12; // Hot gas temperature
	  $ret["HotGas"] = LWZ_DecodeWord(substr($lwzdata,$i,1),substr($lwzdata,$i+1,1)) / 10;

	  $i = 14; // DHW temperature
	  $ret["DHWTemperature"] = LWZ_DecodeWord(substr($lwzdata,$i,1),substr($lwzdata,$i+1,1)) / 10;

	  $i = 16; // Flow HC2 temperature
	  $ret["FlowHC2Temperature"] = LWZ_DecodeWord(substr($lwzdata,$i,1),substr($lwzdata,$i+1,1)) / 10;

	  $i = 18; // Inside temperature
	  $ret["InsideTemperature"] = LWZ_DecodeWord(substr($lwzdata,$i,1),substr($lwzdata,$i+1,1)) / 10;

	  $i = 20; // Evaporator temperature
	  $ret["EvaporatorTemperature"] = LWZ_DecodeWord(substr($lwzdata,$i,1),substr($lwzdata,$i+1,1)) / 10;

	  $i = 22; // Condenser temperature
	  $ret["CondenserTemperature"] = LWZ_DecodeWord(substr($lwzdata,$i,1),substr($lwzdata,$i+1,1)) / 10;

	  $i = 24; // DHW_Pump
	  $ret["DHW_Pump"] = (ord(substr($lwzdata,$i,1)) & 0x1) / 0x1 == 1? true : false;

	  $i = 24; // HeatingCircuit_Pump
	  $ret["HeatingCircuit_Pump"] = (ord(substr($lwzdata,$i,1)) & 0x2) / 0x2 == 1? true : false;

	  $i = 24; // Solar_Pump
	  $ret["Solar_Pump"] = (ord(substr($lwzdata,$i,1)) & 0x8) / 0x8 == 1? true : false;

	  $i = 24; // MixerOpen
	  $ret["MixerOpen"] = (ord(substr($lwzdata,$i,1)) & 0x10) / 0x10 == 1? true : false;

	  $i = 24; // MixerClosed
	  $ret["MixerClosed"] = (ord(substr($lwzdata,$i,1)) & 0x20) / 0x20 == 1? true : false;

	  $i = 24; // HeatPipeValve
	  $ret["HeatPipeValve"] = (ord(substr($lwzdata,$i,1)) & 0x40) / 0x40 == 1? true : false;

	  $i = 24; // DiverterValve
	  $ret["DiverterValve"] = (ord(substr($lwzdata,$i,1)) & 0x80) / 0x80 == 1? true : false;

	  $i = 25; // Compressor
	  $ret["Compressor"] = (ord(substr($lwzdata,$i,1)) & 0x8) / 0x8 == 1? true : false;
	  
	  $i = 25; // BoosterStage3
	  $ret["BoosterStage3"] = (ord(substr($lwzdata,$i,1)) & 0x10) / 0x10 == 1? true : false;

	  $i = 25; // BoosterStage2
	  $ret["BoosterStage2"] = (ord(substr($lwzdata,$i,1)) & 0x20) / 0x20 == 1? true : false;

	  $i = 25; // BoosterStage1
	  $ret["BoosterStage1"] = (ord(substr($lwzdata,$i,1)) & 0x40) / 0x40 == 1? true : false;

	  $i = 26; // HighPressureSensor
	  $ret["HighPressureSensor"] = (ord(substr($lwzdata,$i,1)) & 0x1) / 0x1 == 1? true : false;

	  $i = 26; // LowPressureSensor
	  $ret["LowPressureSensor"] = (ord(substr($lwzdata,$i,1)) & 0x2) / 0x2 == 1? true : false;

	  $i = 26; // EvaporatorIceMonitor
	  $ret["EvaporatorIceMonitor"] = (ord(substr($lwzdata,$i,1)) & 0x4) / 0x4 == 1? true : false;

	  $i = 26; // SignalAnode
	  $ret["SignalAnode"] = (ord(substr($lwzdata,$i,1)) & 0x8) / 0x8 == 1? true : false;

	  $i = 26; // EVU_release
	  $ret["EVU_release"] = (ord(substr($lwzdata,$i,1)) & 0x10) / 0x10 == 1? true : false;

	  $i = 26; // OvenFireplace
	  $ret["OvenFireplace"] = (ord(substr($lwzdata,$i,1)) & 0x20) / 0x20 == 1? true : false;

	  $i = 26; // STB
	  $ret["STB"] = (ord(substr($lwzdata,$i,1)) & 0x40) / 0x40 == 1? true : false;


	  $i = 27; // Extractor speed set
	  $ret["ExtractorSpeedSet"] = LWZ_DecodeWord(substr($lwzdata,$i,1),substr($lwzdata,$i+1,1)) / 10;

	  $i = 29; // Ventilator speed set
	  $ret["VentilatorSpeedSet"] = LWZ_DecodeWord(substr($lwzdata,$i,1),substr($lwzdata,$i+1,1)) / 10;

	  $i = 31; // Expelled air speed set
	  $ret["ExpelledAirSpeedSet"] = LWZ_DecodeWord(substr($lwzdata,$i,1),substr($lwzdata,$i+1,1)) / 10;

	  $i = 33; // Extractor speed actual
	  $ret["ExtractorSpeedActual"] = LWZ_DecodeWord(substr($lwzdata,$i,1),substr($lwzdata,$i+1,1));

	  $i = 35; // Ventilator speed actual
	  $ret["VentilatorSpeedActual"] = LWZ_DecodeWord(substr($lwzdata,$i,1),substr($lwzdata,$i+1,1));

	  $i = 37; // Expelled air speed actual
	  $ret["ExpelledAirSpeedActual"] = LWZ_DecodeWord(substr($lwzdata,$i,1),substr($lwzdata,$i+1,1));


	  $i = 39; // OutsideTemperatureFiltered 
	  $ret["OutsideTemperatureFiltered"] = LWZ_DecodeWord(substr($lwzdata,$i,1),substr($lwzdata,$i+1,1)) / 10;

	  $i = 41; // Relative Humidity
	  $ret["RelativeHumidity"] = LWZ_DecodeWord(substr($lwzdata,$i,1),substr($lwzdata,$i+1,1)) / 10;

  } else {
  		LWZ_SendCommand("get",chr(0xFB));
  }
  return $ret;
}

function LWZ_GetHeatValues($lwzdata = "")
{
  $ret = false;
  if(strlen($lwzdata) > 0) {
	  $ret = array();

	  $i = 10; // Integral Heat
	  $ret["IntegralHeat"] = LWZ_DecodeWord(substr($lwzdata,$i,1),substr($lwzdata,$i+1,1));

	  $i = 14; // Heat set temperature
	  $ret["HeatSetTemperature"] = LWZ_DecodeWord(substr($lwzdata,$i,1),substr($lwzdata,$i+1,1)) / 10;

	  $i = 16; // Heat temperature
	  $ret["HeatTemperature"] = LWZ_DecodeWord(substr($lwzdata,$i,1),substr($lwzdata,$i+1,1)) / 10;

	  $i = 21; // Mode: 1 = Winter, 2 = Summer
	  $ret["Mode"] = LWZ_DecodeWord(chr(0x0),substr($lwzdata,$i,1));

	  $i = 24; // Integral Switch
	  $ret["IntegralSwitch"] = LWZ_DecodeWord(substr($lwzdata,$i,1),substr($lwzdata,$i+1,1));

	  $i = 30; // Room set temperature
	  $ret["RoomSetTemperature"] = LWZ_DecodeWord(substr($lwzdata,$i,1),substr($lwzdata,$i+1,1)) / 10;

  } else {
  		LWZ_SendCommand("get",chr(0xF4));
  }
  return $ret;
}

function LWZ_GetVersion($lwzdata = "")
{
  $version = false;
  if(strlen($lwzdata) > 0) {
		$i = 4; // Version
		$version = LWZ_DecodeWord(substr($lwzdata,$i,1),substr($lwzdata,$i+1,1))/100;
  } else {
		LWZ_SendCommand("get",chr(0xFD));
  }
  return $version;
}

function LWZ_GetOperatingMode($lwzdata = "")
{
  $opmode = false;
  if(strlen($lwzdata) > 0) {
		$i = 6;
		$opmode = LWZ_DecodeWord(substr($lwzdata,$i+1,1),substr($lwzdata,$i,1));
  } else {
		LWZ_SendCommand("get",chr(0x0A).chr(0x01).chr(0x12));
  }
  return $opmode;
}

function LWZ_SetOperatingMode($opmode)
{
	LWZ_SendCommand("set",chr(0x0A).chr(0x01).chr(0x12).chr($opmode));
}

function LWZ_SetFanStage1_Airflow_Inlet($val)
{
	LWZ_SendCommand("set",chr(0x0A).chr(0x05).chr(0x76).LWZ_EncodeWord($val));
}

function LWZ_SetFanStage2_Airflow_Inlet($val)
{
	LWZ_SendCommand("set",chr(0x0A).chr(0x05).chr(0x77).LWZ_EncodeWord($val));
}

function LWZ_SetFanStage3_Airflow_Inlet($val)
{
	LWZ_SendCommand("set",chr(0x0A).chr(0x05).chr(0x78).LWZ_EncodeWord($val));
}

function LWZ_SetFanStage1_Airflow_Outlet($val)
{
	LWZ_SendCommand("set",chr(0x0A).chr(0x05).chr(0x79).LWZ_EncodeWord($val));
}

function LWZ_SetFanStage2_Airflow_Outlet($val)
{
	LWZ_SendCommand("set",chr(0x0A).chr(0x05).chr(0x7A).LWZ_EncodeWord($val));
}
function LWZ_SetFanStage3_Airflow_Outlet($val)
{
	LWZ_SendCommand("set",chr(0x0A).chr(0x05).chr(0x7B).LWZ_EncodeWord($val));
}

function LWZ_GetPassiveCooling($lwzdata = "")
{
  $opmode = false;
  if(strlen($lwzdata) > 0) {
		$i = 6;
		$opmode = LWZ_DecodeWord(substr($lwzdata,$i+1,1),substr($lwzdata,$i,1));
  } else {
		LWZ_SendCommand("get",chr(0x0A).chr(0x05).chr(0x75));
  }
  return $opmode;
}

function LWZ_SetPassiveCooling($mode)
{
	LWZ_SendCommand("set",chr(0x0A).chr(0x05).chr(0x75).chr($mode));
}

?>
